import os
from collections import defaultdict
import numpy as np

import matplotlib

if not os.environ.get("DISPLAY"):
    # Use non-interactive Agg backend
    matplotlib.use("Agg")
import matplotlib.pyplot as plt

from random import choice, uniform

from dark import ncbidb


def basePlotter(blastHits, title):
    """
    Plot the reads and the subject, so that bases in the reads which are
    different from the subject are shown. Else a '.' is shown.
    like so:
    subject_gi  ATGCGTACGTACGACACC
    read_1         A......TTC..T

    @param blastHits: A L{dark.blast.BlastHits} instance.
    @param title: A C{str} sequence title that was matched by BLAST. We plot
        the reads that matched this title.
    """
    result = []
    params = blastHits.plotParams
    assert params is not None, (
        "Oops, it looks like you forgot to run " "computePlotInfo."
    )

    sequence = ncbidb.getSequence(title, blastHits.records.blastDb)
    subject = sequence.seq
    gi = title.split("|")[1]
    sub = "%s\t \t \t%s" % (gi, subject)
    result.append(sub)

    plotInfo = blastHits.titles[title]["plotInfo"]
    assert plotInfo is not None, (
        "Oops, it looks like you forgot to run " "computePlotInfo."
    )

    items = plotInfo["items"]
    count = 0
    for item in items:
        count += 1
        hsp = item["hsp"]
        queryTitle = blastHits.fasta[item["readNum"]].id
        # If the product of the subject and query frame values is +ve,
        # then they're either both +ve or both -ve, so we just use the
        # query as is. Otherwise, we need to reverse complement it.
        if item["frame"]["subject"] * item["frame"]["query"] > 0:
            query = blastHits.fasta[item["readNum"]].seq
            reverse = False
        else:
            # One of the subject or query has negative sense.
            query = blastHits.fasta[item["readNum"]].reverse_complement().seq
            reverse = True
        query = query.upper()
        queryStart = hsp["queryStart"]
        subjectStart = hsp["subjectStart"]
        queryEnd = hsp["queryEnd"]
        subjectEnd = hsp["subjectEnd"]

        # Before comparing the read to the subject, make a string of the
        # same length as the subject, which contains the read and
        # has ' ' where the read does not match.
        # 3 parts need to be taken into account:
        # 1) the left offset (if the query doesn't stick out to the left)
        # 2) the query. if the frame is -1, it has to be reversed.
        # The query consists of 3 parts: left, middle (control for gaps)
        # 3) the right offset

        # Do part 1) and 2).
        if queryStart < 0:
            # The query is sticking out to the left.
            leftQuery = ""
            if subjectStart == 0:
                # The match starts at the first base of the subject.
                middleLeftQuery = ""
            else:
                # The match starts into the subject.
                # Determine the length of the not matching query
                # part to the left.
                leftOffset = -1 * queryStart
                rightOffset = subjectStart + leftOffset
                middleLeftQuery = query[leftOffset:rightOffset]
        else:
            # The query is not sticking out to the left
            # make the left offset.
            leftQuery = queryStart * " "

            leftQueryOffset = subjectStart - queryStart
            middleLeftQuery = query[:leftQueryOffset]

        # Do part 3).
        # Disregard gaps in subject while adding.
        matchQuery = item["origHsp"].query
        matchSubject = item["origHsp"].sbjct
        index = 0
        mid = ""
        for item in range(len(matchQuery)):
            if matchSubject[index] != " ":
                mid += matchQuery[index]
            index += 1
        # if the query has been reversed, turn the matched part around
        if reverse:
            rev = ""
            toReverse = mid
            reverseDict = {
                " ": " ",
                "-": "-",
                "A": "T",
                "T": "A",
                "C": "G",
                "G": "C",
                ".": ".",
                "N": "N",
            }
            for item in toReverse:
                newItem = reverseDict[item]
                rev += newItem
            mid = rev[::-1]

        middleQuery = middleLeftQuery + mid

        # add right not-matching part of the query
        rightQueryOffset = queryEnd - subjectEnd
        rightQuery = query[-rightQueryOffset:]
        middleQuery += rightQuery

        read = leftQuery + middleQuery

        # do part 3)
        offset = len(subject) - len(read)
        # if the read is sticking out to the right
        # chop it off
        if offset < 0:
            read = read[:offset]
        # if it's not sticking out, fill the space with ' '
        elif offset > 0:
            read += offset * " "

        # compare the subject and the read, make a string
        # called 'comparison', which contains a '.' if the bases
        # are equal and the letter of the read if they are not.
        comparison = ""
        for readBase, subjectBase in zip(read, subject):
            if readBase == " ":
                comparison += " "
            elif readBase == subjectBase:
                comparison += "."
            elif readBase != subjectBase:
                comparison += readBase
            index += 1
        que = "%s \t %s" % (queryTitle, comparison)
        result.append(que)

        # sanity checks
        assert len(comparison) == len(subject), "%d != %d" % (
            len(comparison),
            len(subject),
        )

        index = 0
        if comparison[index] == " ":
            index += 1
        else:
            start = index - 1
            assert start == queryStart or start == -1, "%s != %s or %s != -1" % (
                start,
                queryStart,
                start,
            )

    return result


def getAPOBECFrequencies(dotAlignment, orig, new, pattern):
    """
    Gets mutation frequencies if they are in a certain pattern.

    @param dotAlignment: result from calling basePlotter
    @param orig: A C{str}, naming the original base
    @param new: A C{str}, what orig was mutated to
    @param pattern: A C{str}m which pattern we're looking for
        (must be one of 'cPattern', 'tPattern')
    """
    cPattern = [
        "ACA",
        "ACC",
        "ACG",
        "ACT",
        "CCA",
        "CCC",
        "CCG",
        "CCT",
        "GCA",
        "GCC",
        "GCG",
        "GCT",
        "TCA",
        "TCC",
        "TCG",
        "TCT",
    ]
    tPattern = [
        "ATA",
        "ATC",
        "ATG",
        "ATT",
        "CTA",
        "CTC",
        "CTG",
        "CTT",
        "GTA",
        "GTC",
        "GTG",
        "GTT",
        "TTA",
        "TTC",
        "TTG",
        "TTT",
    ]
    # choose the right pattern
    if pattern == "cPattern":
        patterns = cPattern
        middleBase = "C"
    else:
        patterns = tPattern
        middleBase = "T"
    # generate the freqs dict with the right pattern
    freqs = defaultdict(int)
    for pattern in patterns:
        freqs[pattern] = 0
    # get the subject sequence from dotAlignment
    subject = dotAlignment[0].split("\t")[3]
    # exclude the subject from the dotAlignment, so just the queries
    # are left over
    queries = dotAlignment[1:]
    for item in queries:
        query = item.split("\t")[1]
        index = 0
        for queryBase in query:
            qBase = query[index]
            sBase = subject[index]
            if qBase == new and sBase == orig:
                try:
                    plusSb = subject[index + 1]
                    minusSb = subject[index - 1]
                except IndexError:
                    plusSb = "end"
                motif = "%s%s%s" % (minusSb, middleBase, plusSb)
                if motif in freqs:
                    freqs[motif] += 1
            index += 1

    return freqs


def getCompleteFreqs(blastHits):
    """
    Make a dictionary which collects all mutation frequencies from
    all reads.
    Calls basePlotter to get dotAlignment, which is passed to
    getAPOBECFrequencies with the respective parameter, to collect
    the frequencies.

    @param blastHits: A L{dark.blast.BlastHits} instance.
    """
    allFreqs = {}
    for title in blastHits.titles:
        allFreqs[title] = {
            "C>A": {},
            "C>G": {},
            "C>T": {},
            "T>A": {},
            "T>C": {},
            "T>G": {},
        }
        basesPlotted = basePlotter(blastHits, title)
        for mutation in allFreqs[title]:
            orig = mutation[0]
            new = mutation[2]
            if orig == "C":
                pattern = "cPattern"
            else:
                pattern = "tPattern"
            freqs = getAPOBECFrequencies(basesPlotted, orig, new, pattern)
            allFreqs[title][mutation] = freqs
        numberOfReads = len(blastHits.titles[title]["plotInfo"]["items"])
        allFreqs[title]["numberOfReads"] = numberOfReads
        allFreqs[title]["bitScoreMax"] = blastHits.titles[title]["plotInfo"][
            "bitScoreMax"
        ]
    return allFreqs


def makeFrequencyGraph(
    allFreqs,
    title,
    substitution,
    pattern,
    color="blue",
    createFigure=True,
    showFigure=True,
    readsAx=False,
):
    """
    For a title, make a graph showing the frequencies.

    @param allFreqs: result from getCompleteFreqs
    @param title: A C{str}, title of virus of which frequencies should be
        plotted.
    @param substitution: A C{str}, which substitution should be plotted;
        must be one of 'C>A', 'C>G', 'C>T', 'T>A', 'T>C', 'T>G'.
    @param pattern: A C{str}, which pattern we're looking for ( must be
        one of 'cPattern', 'tPattern')
    @param color: A C{str}, color of bars.
    @param createFigure: If C{True}, create a figure.
    @param showFigure: If C{True}, show the created figure.
    @param readsAx: If not None, use this as the subplot for displaying reads.
    """
    cPattern = [
        "ACA",
        "ACC",
        "ACG",
        "ACT",
        "CCA",
        "CCC",
        "CCG",
        "CCT",
        "GCA",
        "GCC",
        "GCG",
        "GCT",
        "TCA",
        "TCC",
        "TCG",
        "TCT",
    ]
    tPattern = [
        "ATA",
        "ATC",
        "ATG",
        "ATT",
        "CTA",
        "CTC",
        "CTG",
        "CTT",
        "GTA",
        "GTC",
        "GTG",
        "GTT",
        "TTA",
        "TTC",
        "TTG",
        "TTT",
    ]

    # choose the right pattern
    if pattern == "cPattern":
        patterns = cPattern
    else:
        patterns = tPattern

    fig = plt.figure(figsize=(10, 10))
    ax = readsAx or fig.add_subplot(111)
    # how many bars
    N = 16
    ind = np.arange(N)
    width = 0.4
    # make a list in the right order, so that it can be plotted easily
    divisor = allFreqs[title]["numberOfReads"]
    toPlot = allFreqs[title][substitution]
    index = 0
    data = []
    for item in patterns:
        newData = toPlot[patterns[index]] / divisor
        data.append(newData)
        index += 1
    # create the bars
    ax.bar(ind, data, width, color=color)
    maxY = np.max(data) + 5
    # axes and labels
    if createFigure:
        title = title.split("|")[4][:50]
        ax.set_title("%s \n %s" % (title, substitution), fontsize=20)
        ax.set_ylim(0, maxY)
        ax.set_ylabel("Absolute Number of Mutations", fontsize=16)
        ax.set_xticks(ind + width)
        ax.set_xticklabels(patterns, rotation=45, fontsize=8)
    if createFigure is False:
        ax.set_xticks(ind + width)
        ax.set_xticklabels(patterns, rotation=45, fontsize=0)
    else:
        if showFigure:
            plt.show()
    return maxY


def makeFrequencyPanel(allFreqs, patientName):
    """
    For a title, make a graph showing the frequencies.

    @param allFreqs: result from getCompleteFreqs
    @param patientName: A C{str}, title for the panel
    """
    titles = sorted(
        iter(allFreqs), key=lambda title: (allFreqs[title]["bitScoreMax"], title)
    )

    origMaxY = 0
    cols = 6
    rows = len(allFreqs)
    figure, ax = plt.subplots(rows, cols, squeeze=False)
    substitutions = ["C>A", "C>G", "C>T", "T>A", "T>C", "T>G"]
    colors = ["blue", "black", "red", "yellow", "green", "orange"]

    for i, title in enumerate(titles):
        for index in range(6):
            for subst in allFreqs[str(title)]:
                substitution = substitutions[index]
                print(i, index, title, "substitution", substitutions[index])
                if substitution[0] == "C":
                    pattern = "cPattern"
                else:
                    pattern = "tPattern"
                maxY = makeFrequencyGraph(
                    allFreqs,
                    title,
                    substitution,
                    pattern,
                    color=colors[index],
                    createFigure=False,
                    showFigure=False,
                    readsAx=ax[i][index],
                )
                if maxY > origMaxY:
                    origMaxY = maxY

            # add title for individual plot.
            # if used for other viruses, this will have to be adapted.
            if index == 0:
                gi = title.split("|")[1]
                titles = title.split(" ")
                try:
                    typeIndex = titles.index("type")
                except ValueError:
                    typeNumber = "gi: %s" % gi
                else:
                    typeNumber = titles[typeIndex + 1]

                ax[i][index].set_ylabel(
                    (
                        "Type %s \n maxBitScore: %s"
                        % (typeNumber, allFreqs[title]["bitScoreMax"])
                    ),
                    fontsize=10,
                )
            # add xAxis tick labels
            if i == 0:
                ax[i][index].set_title(substitution, fontsize=13)
            if i == len(allFreqs) - 1 or i == (len(allFreqs) - 1) / 2:
                if index < 3:
                    pat = [
                        "ACA",
                        "ACC",
                        "ACG",
                        "ACT",
                        "CCA",
                        "CCC",
                        "CCG",
                        "CCT",
                        "GCA",
                        "GCC",
                        "GCG",
                        "GCT",
                        "TCA",
                        "TCC",
                        "TCG",
                        "TCT",
                    ]
                else:
                    pat = [
                        "ATA",
                        "ATC",
                        "ATG",
                        "ATT",
                        "CTA",
                        "CTC",
                        "CTG",
                        "CTT",
                        "GTA",
                        "GTC",
                        "GTG",
                        "GTT",
                        "TTA",
                        "TTC",
                        "TTG",
                        "TTT",
                    ]
                ax[i][index].set_xticklabels(pat, rotation=45, fontsize=8)

    # make Y-axis equal
    for i, title in enumerate(allFreqs):
        for index in range(6):
            a = ax[i][index]
            a.set_ylim([0, origMaxY])
    # add title of whole panel
    figure.suptitle("Mutation Signatures in %s" % patientName, fontsize=20)
    figure.set_size_inches(5 * cols, 3 * rows, forward=True)
    figure.show()

    return allFreqs


def mutateString(original, n, replacements="acgt"):
    """
    Mutate C{original} in C{n} places with chars chosen from C{replacements}.

    @param original: The original C{str} to mutate.
    @param n: The C{int} number of locations to mutate.
    @param replacements: The C{str} of replacement letters.

    @return: A new C{str} with C{n} places of C{original} mutated.
    @raises ValueError: if C{n} is too high, or C{replacement} contains
        duplicates, or if no replacement can be made at a certain locus
        because C{replacements} is of length one, or if C{original} is of
        zero length.
    """
    if not original:
        raise ValueError("Empty original string passed.")

    if n > len(original):
        raise ValueError(
            "Cannot make %d mutations in a string of length %d" % (n, len(original))
        )

    if len(replacements) != len(set(replacements)):
        raise ValueError("Replacement string contains duplicates")

    if len(replacements) == 1 and original.find(replacements) != -1:
        raise ValueError("Impossible replacement")

    result = list(original)
    length = len(original)

    for offset in range(length):
        if uniform(0.0, 1.0) < float(n) / (length - offset):
            # Mutate.
            while True:
                new = choice(replacements)
                if new != result[offset]:
                    result[offset] = new
                    break
            n -= 1
            if n == 0:
                break

    return "".join(result)



#!/usr/bin/env python

from setuptools import setup


# Modified from http://stackoverflow.com/questions/2058802/
# how-can-i-get-the-version-defined-in-setup-py-setuptools-in-my-package
def version():
    import os
    import re

    init = os.path.join("dark", "__init__.py")
    with open(init) as fp:
        initData = fp.read()
    match = re.search(r"^__version__ = ['\"]([^'\"]+)['\"]", initData, re.M)
    if match:
        return match.group(1)
    else:
        raise RuntimeError("Unable to find version string in %r." % init)


# Explicitly list bin scripts to be installed, seeing as I have a few local
# bin files that are not (yet) part of the distribution.
scripts = [
    "bin/aa-info.py",
    "bin/aa-to-dna.py",
    "bin/aa-to-properties.py",
    "bin/adaptor-distances.py",
    "bin/alignment-panel-civ.py",
    "bin/alignments-per-read.py",
    "bin/bit-score-to-e-value.py",
    "bin/cat-json-blast-records.py",
    "bin/check-fasta-json-blast-consistency.py",
    "bin/codon-distance.py",
    "bin/compare-consensuses.py",
    "bin/compare-sequences.py",
    "bin/convert-blast-xml-to-json.py",
    "bin/convert-diamond-to-json.py",
    "bin/convert-diamond-to-sam.py",
    "bin/convert-sam-to-fastq.sh",
    "bin/create-newick-relabeling-output.py",
    "bin/dark-matter-version.py",
    "bin/describe-protein-database.py",
    "bin/dna-to-aa.py",
    "bin/download-genbank.sh",
    "bin/download-refseq-viral-fasta.sh",
    "bin/download-refseq-viral-gbff.sh",
    "bin/e-value-to-bit-score.py",
    "bin/extract-ORFs.py",
    "bin/fasta-base-indices.py",
    "bin/fasta-count.py",
    "bin/fasta-count-chars.py",
    "bin/fasta-coverage.py",
    "bin/fasta-diff.sh",
    "bin/fasta-find.py",
    "bin/fasta-identity-table.py",
    "bin/fasta-ids.py",
    "bin/fasta-join.py",
    "bin/fasta-lengths.py",
    "bin/fasta-match-offsets.py",
    "bin/fasta-sequences.py",
    "bin/fasta-sort.py",
    "bin/fasta-split-by-id.py",
    "bin/fasta-split.py",
    "bin/fasta-subset.py",
    "bin/fasta-subtraction.py",
    "bin/fasta-to-phylip.py",
    "bin/fasta-translate.py",
    "bin/fasta-variable-sites.py",
    "bin/fastq-set-quality.py",
    "bin/filter-fasta-by-complexity.py",
    "bin/filter-fasta-by-taxonomy.py",
    "bin/filter-fasta.py",
    "bin/filter-hits-to-fasta.py",
    "bin/filter-reads-alignments.py",
    "bin/filter-sam.py",
    "bin/format-fasta.py",
    "bin/genbank-grep.py",
    "bin/genome-protein-summary.py",
    "bin/get-features.py",
    "bin/get-hosts.py",
    "bin/get-reads.py",
    "bin/get-taxonomy.py",
    "bin/graph-evalues.py",
    "bin/local-align.py",
    "bin/make-consensus.py",
    "bin/make-fasta-database.py",
    "bin/make-protein-database.py",
    "bin/ncbi-fetch-id.py",
    "bin/newick-to-ascii.py",
    "bin/noninteractive-alignment-panel.py",
    "bin/parse-genbank-flat-file.py",
    "bin/plot-references-by-inter-read-distance.py",
    "bin/position-summary.py",
    "bin/pre-commit.sh",
    "bin/print-read-lengths.py",
    "bin/proteins-to-pathogens.py",
    "bin/proteins-to-pathogens-civ.py",
    "bin/randomize-fasta.py",
    "bin/read-blast-json.py",
    "bin/read-blast-xml.py",
    "bin/reference-read-scores-to-JSON.py",
    "bin/relabel-newick-tree.py",
    "bin/run-bwa.py",
    "bin/run-bowtie2.py",
    "bin/sam-coverage.py",
    "bin/sam-coverage-depth.py",
    "bin/sam-to-fasta-alignment.py",
    "bin/sam-reference-read-counts.py",
    "bin/sam-references.py",
    "bin/sff-to-fastq.py",
    "bin/simple-consensus.py",
    "bin/split-fasta-by-adaptors.py",
    "bin/subset-protein-database.py",
    "bin/summarize-fasta-bases.py",
    "bin/summarize-reads.py",
    "bin/translate.py",
    "bin/trim-primers.py",
    "bin/trim-reads.py",
    "bin/write-htcondor-job-spec.py",
]

setup(
    name="dark-matter",
    version=version(),
    packages=["dark", "dark.blast", "dark.diamond", "dark.civ"],
    url="https://github.com/acorg/dark-matter",
    download_url="https://github.com/acorg/dark-matter",
    author="Terry Jones, Barbara Mühlemann, Tali Veith, Sophie Mathias, Udo Gieraths",
    author_email="tcj25@cam.ac.uk",
    keywords=["virus discovery"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    license="MIT",
    description=(
        "Python classes and utility scripts for working with genetic sequence data."
    ),
    scripts=scripts,
    install_requires=[
        "biopython>=1.71",
        "bz2file>=0.98",
        "cachetools>=3.1.0",
        "edlib>=1.3.9",
        "Cython>=0.29.16",
        "gb2seq>=0.2.4",
        "ipython>=3.1.0",
        "matplotlib>=1.4.3",
        "mysql-connector-python==8.0.19",
        "numpy>=1.14.2",
        "progressbar2>=4.0.0",
        "pysam>=0.15.2",
        "pyzmq>=14.3.1",
        "requests>=2.18.4",
        "simplejson>=3.5.3",
        "six>=1.11.0",
    ],
)



#!/usr/bin/env python

"""
Aligns each FASTA sequence in one file to each sequence in another.
"""

import argparse

from dark.fasta import FastaReads
from dark.local_align import LocalAlignment


parser = argparse.ArgumentParser(
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    description=(
        "Given two FASTA files, perform a local Smith Waterman alignment "
        "on all the sequences in one file against all those in the other."
    ),
)

parser.add_argument("--fastaFile1", required=True, help="The first FASTA file.")

parser.add_argument("--fastaFile2", required=True, help="The second FASTA file.")

parser.add_argument("--matchScore", type=int, default=1, help="The match score.")

parser.add_argument("--mismatchScore", type=int, default=-1, help="The mismatch score.")

parser.add_argument("--gapOpenScore", type=int, default=-1, help="The gap open score.")

parser.add_argument(
    "--gapExtendScore", type=int, default=-1, help="The gap extend score."
)

parser.add_argument(
    "--gapExtendDecay", type=float, default=0.0, help="The gap extend decay."
)

args = parser.parse_args()

for seq1 in FastaReads(args.fastaFile1):
    for seq2 in FastaReads(args.fastaFile2):
        alignment = LocalAlignment(
            seq1,
            seq2,
            match=args.matchScore,
            mismatch=args.mismatchScore,
            gap=args.gapOpenScore,
            gapExtend=args.gapExtendScore,
            gapExtendDecay=args.gapExtendDecay,
        )

        print(alignment.createAlignment(resultFormat=str))



from collections import defaultdict

TRANSITIONS = {("A", "G"), ("G", "A"), ("C", "T"), ("T", "C")}


def findDistance(co1, co2):
    """
    Find the distance between two codons.

    @param co1: A C{str} of length three.
    @param co2: A C{str} of length three.
    @return: An C{int} distance in [0, 3].
    """
    return (co1[0] != co2[0]) + (co1[1] != co2[1]) + (co1[2] != co2[2])


def codonInformation(codons1, codons2, countTransitions=False):
    """
    Take two C{list} of codons, and returns information about the min and
    max distance between them.

    @param codon1: a C{list} of codons.
    @param codon2: a C{list} of codons.
    @param countTransitions: A C{bool}. If C{True}, the tuples in the C{list}
        values of the returned C{dict} will also contain a count of the number
        of transitions (as opposed to transversions) between the pair of codons.

    @return: a dict whose keys are C{int} distances and whose values are tuples
        of pairs of codons (and the number of transitions involved, if
        C{countTransitions} is C{True})
    """
    result = defaultdict(list)
    for c1 in codons1:
        for c2 in codons2:
            distance = findDistance(c1, c2)
            if countTransitions:
                transitions = sum((a, b) in TRANSITIONS for a, b in zip(c1, c2))
                result[distance].append((c1, c2, transitions))
            else:
                result[distance].append((c1, c2))

    return result



#!/usr/bin/env python

import sys
from Bio import SeqIO

from dark import fasta

if __name__ == "__main__":
    if len(sys.argv) == 1:
        print(
            "Usage: %s 1.fasta, 2.fasta, ... > seq.fasta" % (sys.argv[0]),
            file=sys.stderr,
        )
        sys.exit(1)
    else:
        reads = fasta.fastaSubtract(map(open, sys.argv[1:]))
        SeqIO.write(reads, sys.stdout, "fasta")



#!/usr/bin/env python

import sys

from dark.aa import find
from dark.aa import CODONS

from dark.aa import AA_LETTERS, ALL_PROPERTIES, PROPERTY_NAMES

args = AA_LETTERS if len(sys.argv) == 1 else sys.argv[1:]
error = False

for arg in args:
    aas = list(find(arg))
    if aas:
        for aa in aas:
            print(aa.name)
            print("  3-letter abbreviation: %s" % aa.abbrev3)
            print("  1-letter abbreviation: %s" % aa.abbrev1)
            print("  Codons: %s" % ", ".join(sorted(aa.codons)))

            properties = []
            print("  Properties:", end=" ")
            for prop in ALL_PROPERTIES:
                if aa.properties & prop:
                    properties.append(PROPERTY_NAMES[prop])
            print(", ".join(properties))

            print("  Property details:")
            for propertyDetail, value in aa.propertyDetails.items():
                print("    %s: %s" % (propertyDetail, value))
    else:
        error = True
        print("Unknown amino acid or codon: %s" % arg, file=sys.stderr)

if error:
    print("Valid arguments are: %s." % list(CODONS), file=sys.stderr)



#!/usr/bin/env python

from dark.reads import addFASTACommandLineOptions, parseFASTACommandLineOptions


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description=(
            "Given FASTA on stdin, write the sequence ids and " "lengths to stdout."
        )
    )

    addFASTACommandLineOptions(parser)
    args = parser.parse_args()
    reads = parseFASTACommandLineOptions(args)

    for read in reads:
        print(read.id)



#!/usr/bin/env python

# See https://samtools.github.io/hts-specs/SAMv1.pdf for the SAM file
# format specification.

import sys
import argparse
from os.path import join
from resource import getrlimit, RLIMIT_NOFILE
from functools import partial

from dark.diamond.conversion import FIELDS
from dark.diamond.sam import SimpleDiamondSAMWriter, PerReferenceDiamondSAMWriter
from dark.proteins import SqliteIndex

parser = argparse.ArgumentParser(
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    description=(
        "Convert DIAMOND tabular format to SAM. The DIAMOND "
        "invocation *must* include --outfmt 6 %s" % FIELDS
    ),
)

parser.add_argument(
    "--printFields",
    default=False,
    action="store_true",
    help=(
        "Print the field names in the order that they must be given to "
        "diamond --outfmt 6 to produce correct input for this script, "
        "then exit."
    ),
)

parser.add_argument(
    "--mappingQuality",
    type=int,
    default=255,
    help=(
        "The mapping quality to use for MAPQ (field 5). The default (255) "
        "indicates that mapping quality information is not available."
    ),
)

parser.add_argument(
    "--ram",
    action="store_true",
    default=False,
    help=(
        "Do not use a temporary file to hold the non-header SAM output. "
        "This will run faster but use more memory since all non-header SAM "
        "output will be stored in RAM and only written out when the full "
        "header can be determined."
    ),
)

parser.add_argument(
    "--keepDescriptions",
    action="store_true",
    default=False,
    help=(
        "Do not discard text after the first space in query or subject "
        "sequence ids. Note that this violates the SAM specification, but "
        "since SAM files are TAB-separated it may not cause problems "
        "downstream, depending on what tools you use on the SAM."
    ),
)

parser.add_argument(
    "--genomesProteinsDatabaseFilename",
    required=True,
    help=(
        "The name of the file containing an sqlite3 database with "
        "information on genomes and proteins, as made by "
        "make-protein-database.py"
    ),
)

parser.add_argument(
    "--outputDirectory",
    default=".",
    help=(
        "The name of the directory to create per-reference SAM files in. "
        "This is only used when --perReferenceOutput is given, otherwise "
        "the output is written to stdout as a single SAM file."
    ),
)

parser.add_argument(
    "--perReferenceOutput",
    action="store_true",
    default=False,
    help=(
        "If given, a separate SAM file is written for matches against each "
        "reference sequence. The files will have names corresponding to "
        "the accession number of the reference, with a .sam suffix "
        "and will be put into the directory specified by "
        "--outputDirectory."
    ),
)

parser.add_argument(
    "--filePointerCacheMaxSize",
    type=int,
    default=getrlimit(RLIMIT_NOFILE)[0] >> 1,
    help=(
        "The maximum size for the file pointer cache. SAM output may "
        "be being written to an arbitrary number of output files so we "
        "need to make sure the OS limit on open files is not exceeded. "
        "The default value is size half the maximum number of open files "
        "will be used."
    ),
)

args = parser.parse_args()

if args.printFields:
    print(FIELDS)
    sys.exit(0)

if 0 > args.mappingQuality > 255:
    raise ValueError("Mapping quality must be between 0 and 255 (inclusive)")

genomesProteins = SqliteIndex(args.genomesProteinsDatabaseFilename)

if args.perReferenceOutput:

    def referenceTitleToBasename(subjectTitle):
        """
        Make a basename for a SAM output file for a DIAMOND subject match.

        @param subjectTitle: A C{str} database subject id. This must have a
            form like "acc|GENBANK|YP_009137150.1|GENBANK|NC_001798.2|..."
            where the accession number of the matched sequence is in the third
            '|'-separated field (which must not contain any characters that
            cannot be used in a filename).
        @return: A C{str} pathname to store the SAM for the reference to.
        """
        return join(args.outputDirectory, genomesProteins.genomeAccession(subjectTitle))

    writer = PerReferenceDiamondSAMWriter(
        genomesProteins,
        mappingQuality=args.mappingQuality,
        ram=args.ram,
        keepDescriptions=args.keepDescriptions,
        baseFilenameFunc=referenceTitleToBasename,
        fpcMaxsize=args.filePointerCacheMaxSize,
    )

    save = writer.save
else:
    writer = SimpleDiamondSAMWriter(
        genomesProteins,
        mappingQuality=args.mappingQuality,
        ram=args.ram,
        keepDescriptions=args.keepDescriptions,
    )

    save = partial(writer.save, filename=sys.stdout)

addMatch = writer.addMatch

for line in sys.stdin:
    addMatch(line)

save()



from unittest import TestCase
from Bio.Seq import Seq

from dark.orfs import findCodons


class TestFindCodons(TestCase):
    """Tests of the findCodons helper."""

    def testNoMatches(self):
        """
        When there are no codons in the sequence, returns an empty list.
        """
        seq = Seq("AAAAAA")
        self.assertEqual([], list(findCodons(seq, set(["ATG", "AGG"]))))

    def testMatchAtStart(self):
        """
        Finds a codon at the start of the sequence.
        """
        seq = Seq("ATGAAA")
        self.assertEqual([0], list(findCodons(seq, set(["ATG", "AGG"]))))

    def testMatchAtEnd(self):
        """
        Finds a codon at the end of the sequence.
        """
        seq = Seq("ATGAAA")
        self.assertEqual([3], list(findCodons(seq, set(["AAA"]))))

    def testMatchMultiple(self):
        """
        Finds multiple codons in the sequence.
        """
        seq = Seq("ATGAAAGGGCCC")
        self.assertEqual([0, 9], list(findCodons(seq, set(["ATG", "CCC"]))))

    def testDoesNotFindOutOfFrameMatches(self):
        """
        Does not find matching codons that are in a non-zero frame in the
        sequence.
        """
        seq = Seq("TATGAAAGGGCCC")
        self.assertEqual([], list(findCodons(seq, set(["ATG", "CCC"]))))
